@@ -10,7 +10,7 @@ use Monitoring::Livestatus::INET;
 use Monitoring::Livestatus::UNIX;
 use Monitoring::Livestatus::MULTI;
 
-our $VERSION = '0.38';
+our $VERSION = '0.46';
 
 
 =head1 NAME
@@ -95,12 +95,23 @@ currently only querys without Columns: Header will result in a warning
 
 =item timeout
 
-set a general timeout. Used for connect and querys, Default 10sec
+set a general timeout. Used for connect and querys, no default
+
+=item query_timeout
+
+set a query timeout. Used for retrieving querys, Default 60sec
+
+=item connect_timeout
+
+set a connect timeout. Used for initial connections, default 5sec
 
 =item use_threads
 
 only used with multiple backend connections.
-Default is to use threads where available.
+Default is to don't threads where available. As threads in perl
+are causing problems with tied resultset and using more memory.
+Querys are usually faster without threads, except for very slow backends
+connections.
 
 =back
 
@@ -127,11 +138,15 @@ sub new {
       "keepalive"                 => 0,       # enable keepalive?
       "errors_are_fatal"          => 1,       # die on errors
       "backend"                   => undef,   # should be keept undef, used internally
-      "timeout"                   => 10,      # timeout for tcp connections
+      "timeout"                   => undef,   # timeout for tcp connections
+      "query_timeout"             => 60,      # query timeout for tcp connections
+      "connect_timeout"           => 5,       # connect timeout for tcp connections
+      "timeout"                   => undef,   # timeout for tcp connections
       "use_threads"               => undef,   # use threads, default is to use threads where available
       "warnings"                  => 1,       # show warnings, for example on querys without Column: Header
       "logger"                    => undef,   # logger object used for statistical informations and errors / warnings
       "deepcopy"                  => undef,   # copy result set to avoid errors with tied structures
+      "disabled"                  => 0,       # if disabled, this peer will not receive any query
     };
 
     for my $opt_key (keys %options) {
@@ -143,6 +158,12 @@ sub new {
         }
     }
 
+    # setting a general timeout?
+    if(defined $self->{'timeout'}) {
+        $self->{'query_timeout'}   = $self->{'timeout'};
+        $self->{'connect_timeout'} = $self->{'timeout'};
+    }
+
     bless $self, $class;
 
     # set our peer(s) from the options
@@ -204,6 +225,7 @@ Always returns true.
 sub do {
     my $self      = shift;
     my $statement = shift;
+    return if $self->{'disabled'};
     $self->_send($statement);
     return(1);
 }
@@ -253,20 +275,14 @@ sub selectall_arrayref {
     my $statement = shift;
     my $opt       = shift;
     my $limit     = shift || 0;
+    return if $self->{'disabled'};
     my $result;
 
     # make opt hash keys lowercase
     $opt = $self->_lowercase_and_verify_options($opt);
 
     if(defined $self->{'logger'}) {
-        my $d = Data::Dumper->new([$opt]);
-        $d->Indent(0);
-        my $optstring = $d->Dump;
-        $optstring =~ s/^\$VAR1\s+=\s+//mx;
-        $optstring =~ s/;$//mx;
-        my $cleanstatement = $statement;
-        $cleanstatement =~ s/\n/\\n/gmx;
-        $self->{'logger'}->debug('selectall_arrayref("'.$cleanstatement.'", '.$optstring.', '.$limit.')')
+        $self->_log_statement($statement, $opt, $limit);
     }
 
     $result = $self->_send($statement, $opt);
@@ -682,6 +698,39 @@ sub marked_bad {
 }
 
 
+########################################
+
+=head2 disable
+
+ $ml->disable()
+
+disables this connection, returns the last state.
+
+=cut
+sub disable {
+    my $self  = shift;
+    my $prev = $self->{'disabled'};
+    $self->{'disabled'} = 1;
+    return $prev;
+}
+
+
+########################################
+
+=head2 enable
+
+ $ml->enable()
+
+enables this connection, returns the last state.
+
+=cut
+sub enable {
+    my $self  = shift;
+    my $prev = $self->{'disabled'};
+    $self->{'disabled'} = 0;
+    return $prev;
+}
+
 ########################################
 # INTERNAL SUBS
 ########################################
@@ -885,9 +934,6 @@ sub _open {
         $self->{'sock'} = $sock;
     }
 
-    # set timeout
-    $sock->timeout($self->{'timeout'}) if defined $sock;
-
     return($sock);
 }
 
@@ -1038,10 +1084,6 @@ sub _socket_error {
     my $message = "\n";
     $message   .= "peer                ".Dumper($self->peer_name);
     $message   .= "statement           ".Dumper($statement);
-    #$message   .= "socket->sockname()  ".Dumper($sock->sockname());
-    #$message   .= "socket->atmark()    ".Dumper($sock->atmark());
-    #$message   .= "socket->error()     ".Dumper($sock->error());
-    #$message   .= "socket->timeout()   ".Dumper($sock->timeout());
     $message   .= "message             ".Dumper($body);
 
     $self->{'logger'}->error($message) if defined $self->{'logger'};
@@ -1221,6 +1263,7 @@ sub _get_error {
     my $codes = {
         '200' => 'OK. Reponse contains the queried data.',
         '201' => 'COMMANDs never return something',
+        '400' => 'The request contains an invalid header.',
         '401' => 'The request contains an invalid header.',
         '402' => 'The request is completely invalid.',
         '403' => 'The request is incomplete.',
@@ -1367,6 +1410,25 @@ sub _lowercase_and_verify_options {
     return($return);
 }
 
+########################################
+sub _log_statement {
+    my $self      = shift;
+    my $statement = shift;
+    my $opt       = shift;
+    my $limit     = shift;
+    my $d = Data::Dumper->new([$opt]);
+    $d->Indent(0);
+    my $optstring = $d->Dump;
+    $optstring =~ s/^\$VAR1\s+=\s+//mx;
+    $optstring =~ s/;$//mx;
+
+    # remove empty lines from statement
+    $statement =~ s/\n+/\n/gmx;
+
+    my $cleanstatement = $statement;
+    $cleanstatement =~ s/\n/\\n/gmx;
+    $self->{'logger'}->debug('selectall_arrayref("'.$cleanstatement.'", '.$optstring.', '.$limit.')')
+}
 
 ########################################
 
