@@ -90,20 +90,12 @@ sub new {
 
     # initialize peer keys
     $self->{'peer_by_key'} = {};
+    $self->{'peer_by_addr'} = {};
     for my $peer (@{$self->{'peers'}}) {
-        $self->{'peer_by_key'}->{$peer->peer_key} = $peer;
+        $self->{'peer_by_key'}->{$peer->peer_key}   = $peer;
+        $self->{'peer_by_addr'}->{$peer->peer_addr} = $peer;
     }
 
-    $self->{'allowed_options'} = {
-        'addpeer'   => 1,
-        'backend'   => 1,
-        'columns'   => 1,
-        'deepcopy'  => 1,
-        'rename'    => 1,
-        'slice'     => 1,
-        'sum'       => 1,
-    };
-
     $self->{'name'} = 'multiple connector' unless defined $self->{'name'};
     $self->{'logger'}->debug('initialized Monitoring::Livestatus::MULTI '.($self->{'use_threads'} ? 'with' : 'without' ).' threads') if defined $self->{'logger'};
 
@@ -132,6 +124,7 @@ sub do {
     return 1;
 }
 
+
 ########################################
 
 =head2 selectall_arrayref
@@ -145,13 +138,22 @@ sub selectall_arrayref {
     my $opts  = $self->_lowercase_and_verify_options($_[1]);
     my $t0    = [gettimeofday];
 
+    if(defined $self->{'logger'}) {
+        $self->_log_statement($_[0], $opts, 0);
+    }
+
     my $return  = $self->_merge_answer($self->_do_on_peers("selectall_arrayref", $opts->{'backends'}, @_));
     my $elapsed = tv_interval ( $t0 );
-    $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for selectall_arrayref() in total') if defined $self->{'logger'};
+    if(defined $self->{'logger'}) {
+        my $total_results = 0;
+        $total_results    = scalar @{$return} if defined $return;
+        $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for selectall_arrayref() in total, results: '.$total_results);
+    }
 
     return $return;
 }
 
+
 ########################################
 
 =head2 selectall_hashref
@@ -172,6 +174,7 @@ sub selectall_hashref {
     return $return;
 }
 
+
 ########################################
 
 =head2 selectcol_arrayref
@@ -192,6 +195,7 @@ sub selectcol_arrayref {
     return $return;
 }
 
+
 ########################################
 
 =head2 selectrow_array
@@ -207,7 +211,7 @@ sub selectrow_array {
     my $t0        = [gettimeofday];
     my @return;
 
-    if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
+    if((defined $opts->{'sum'} and $opts->{'sum'} == 1) or (!defined $opts->{'sum'} and $statement =~ m/^Stats:/mx)) {
         @return = @{$self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_))};
     } else {
         if($self->{'warnings'}) {
@@ -223,6 +227,7 @@ sub selectrow_array {
     return @return;
 }
 
+
 ########################################
 
 =head2 selectrow_arrayref
@@ -238,7 +243,7 @@ sub selectrow_arrayref {
     my $t0        = [gettimeofday];
     my $return;
 
-    if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
+    if((defined $opts->{'sum'} and $opts->{'sum'} == 1) or (!defined $opts->{'sum'} and $statement =~ m/^Stats:/mx)) {
         $return = $self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_));
     } else {
         if($self->{'warnings'}) {
@@ -254,6 +259,7 @@ sub selectrow_arrayref {
     return $return;
 }
 
+
 ########################################
 
 =head2 selectrow_hashref
@@ -271,7 +277,7 @@ sub selectrow_hashref {
 
     my $return;
 
-    if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
+    if((defined $opts->{'sum'} and $opts->{'sum'} == 1) or (!defined $opts->{'sum'} and $statement =~ m/^Stats:/mx)) {
         $return = $self->_sum_answer($self->_do_on_peers("selectrow_hashref", $opts->{'backends'}, @_));
     } else {
         if($self->{'warnings'}) {
@@ -286,6 +292,7 @@ sub selectrow_hashref {
     return $return;
 }
 
+
 ########################################
 
 =head2 selectscalar_value
@@ -303,7 +310,7 @@ sub selectscalar_value {
 
     my $return;
 
-    if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
+    if((defined $opts->{'sum'} and $opts->{'sum'} == 1) or (!defined $opts->{'sum'} and $statement =~ m/^Stats:/mx)) {
         return $self->_sum_answer($self->_do_on_peers("selectscalar_value", $opts->{'backends'}, @_));
     } else {
         if($self->{'warnings'}) {
@@ -320,6 +327,7 @@ sub selectscalar_value {
     return $return;
 }
 
+
 ########################################
 
 =head2 errors_are_fatal
@@ -334,6 +342,7 @@ sub errors_are_fatal {
     return $self->_change_setting('errors_are_fatal', $value);
 }
 
+
 ########################################
 
 =head2 warnings
@@ -348,6 +357,7 @@ sub warnings {
     return $self->_change_setting('warnings', $value);
 }
 
+
 ########################################
 
 =head2 verbose
@@ -423,6 +433,57 @@ sub peer_key {
 }
 
 
+########################################
+
+=head2 disable
+
+ $ml->disable()
+
+disables this connection, returns the last state.
+
+=cut
+sub disable {
+    my $self     = shift;
+    my $peer_key = shift;
+    if(!defined $peer_key) {
+        for my $peer (@{$self->{'peers'}}) {
+            $peer->disable();
+        }
+        return 1;
+    } else {
+        my $peer     = $self->_get_peer_by_key($peer_key);
+        my $prev     = $peer->{'disabled'};
+        $peer->{'disabled'} = 1;
+        return $prev;
+    }
+}
+
+
+########################################
+
+=head2 enable
+
+ $ml->enable()
+
+enables this connection, returns the last state.
+
+=cut
+sub enable {
+    my $self     = shift;
+    my $peer_key = shift;
+    if(!defined $peer_key) {
+        for my $peer (@{$self->{'peers'}}) {
+            $peer->enable();
+        }
+        return 1;
+    } else {
+        my $peer     = $self->_get_peer_by_key($peer_key);
+        my $prev     = $peer->{'disabled'};
+        $peer->{'disabled'} = 0;
+        return $prev;
+    }
+}
+
 ########################################
 # INTERNAL SUBS
 ########################################
@@ -450,6 +511,7 @@ sub _change_setting {
     return $old;
 }
 
+
 ########################################
 sub _start_worker {
     my $self = shift;
@@ -475,6 +537,7 @@ sub _start_worker {
     return;
 }
 
+
 ########################################
 sub _stop_worker {
     # try to kill our threads safely
@@ -486,6 +549,7 @@ sub _stop_worker {
     return;
 }
 
+
 ########################################
 sub _worker_thread {
     local $SIG{'USR1'} = sub { threads->exit(); };
@@ -509,6 +573,7 @@ sub _worker_thread {
     return;
 }
 
+
 ########################################
 sub _do_wrapper {
     my $peer   = shift;
@@ -533,6 +598,7 @@ sub _do_wrapper {
     return $return;
 }
 
+
 ########################################
 sub _do_on_peers {
     my $self        = shift;
@@ -566,8 +632,9 @@ sub _do_on_peers {
             croak("unsupported type for backend: ".ref($backends));
         }
 
-        for my $back (@backends) {
-            push @peers, $self->_get_peer_by_key($back);
+        for my $key (@backends) {
+            my $backend = $self->_get_peer_by_key($key);
+            push @peers, $backend unless $backend->{'disabled'};
         }
     } else {
         # use all backends
@@ -577,13 +644,24 @@ sub _do_on_peers {
     # its faster without threads for only one peer
     if(scalar @peers <= 1) { $use_threads = 0; }
 
+    # if we have limits set, we cannot use threads
+    if(defined $query_options->{'limit_start'}) { $use_threads = 0; }
+
     if($use_threads) {
         # use the threaded variant
-        print("using threads\n") if $self->{'verbose'};
+        $self->{'logger'}->debug('using threads') if defined $self->{'logger'};
 
         my $peers_to_use;
         for my $peer (@peers) {
-            $peers_to_use->{$peer->peer_key} = 1;
+            if($peer->{'disabled'}) {
+                # dont send any query
+            }
+            elsif($peer->marked_bad) {
+                warn($peer->peer_name.' ('.$peer->peer_key.') is marked bad') if $self->{'verbose'};
+            }
+            else {
+                $peers_to_use->{$peer->peer_key} = 1;
+            }
         }
         my $x = 0;
         for my $peer (@{$self->{'peers'}}) {
@@ -598,7 +676,7 @@ sub _do_on_peers {
             $x++;
         }
 
-        for(my $x = 0; $x < scalar @peers; $x++) {
+        for(my $x = 0; $x < scalar keys %{$peers_to_use}; $x++) {
             my $result = $self->{'WorkResults'}->dequeue;
             my $peer   = $self->{'peers'}->[$result->{'peer'}];
             if(defined $result->{'result'}) {
@@ -609,14 +687,26 @@ sub _do_on_peers {
             }
         }
     } else {
-        print("not using threads\n") if $self->{'verbose'};
+        $self->{'logger'}->debug('not using threads') if defined $self->{'logger'};
         for my $peer (@peers) {
-            if($peer->marked_bad) {
+            if($peer->{'disabled'}) {
+                # dont send any query
+            }
+            elsif($peer->marked_bad) {
                 warn($peer->peer_name.' ('.$peer->peer_key.') is marked bad') if $self->{'verbose'};
             } else {
                 my $erg = _do_wrapper($peer, $sub, $self->{'logger'}, @opts);
                 $return->{$peer->peer_key} = $erg->{'data'};
                 push @{$codes{$erg->{'code'}}}, { 'peer' => $peer, 'msg' => $erg->{'msg'} };
+
+                # compute limits
+                if(defined $query_options->{'limit_length'} and $peer->{'meta_data'}->{'result_count'}) {
+                    last;
+                }
+                # set a new start if we had rows already
+                if(defined $query_options->{'limit_start'}) {
+                    $query_options->{'limit_start'} = $query_options->{'limit_start'} - $peer->{'meta_data'}->{'row_count'};
+                }
             }
         }
     }
@@ -628,14 +718,16 @@ sub _do_on_peers {
     my @codes = sort keys %codes;
     if(scalar @codes > 1) {
         # got different results for our backends
-        print "got different result stati: ".Dumper(\%codes) if $self->{'verbose'};
+        if(defined $self->{'logger'}) {
+            $self->{'logger'}->warn("got different result stati: ".Dumper(\%codes));
+        }
     } else {
         # got same result codes for all backend
     }
 
     my $failed = 0;
     my $code = $codes[0];
-    if($code >= 300) {
+    if(defined $code and $code >= 300) {
         $failed = 1;
     }
 
@@ -669,6 +761,7 @@ sub _do_on_peers {
     return($return);
 }
 
+
 ########################################
 sub _merge_answer {
     my $self   = shift;
@@ -699,6 +792,7 @@ sub _merge_answer {
     return($return);
 }
 
+
 ########################################
 sub _sum_answer {
     my $self   = shift;
@@ -738,6 +832,7 @@ sub _sum_answer {
     return $return;
 }
 
+
 ########################################
 sub _clone {
     my $self   = shift;
@@ -771,6 +866,7 @@ sub _clone {
     return $return;
 }
 
+
 ########################################
 sub _get_peer_by_key {
     my $self = shift;
@@ -782,21 +878,19 @@ sub _get_peer_by_key {
     return $self->{'peer_by_key'}->{$key};
 }
 
+
 ########################################
-sub _lowercase_and_verify_options {
+sub _get_peer_by_addr {
     my $self = shift;
-    my $opts = shift;
-    my $return;
+    my $addr = shift;
 
-    for my $key (keys %{$opts}) {
-        if($self->{'warnings'} and !defined $self->{'allowed_options'}->{lc $key}) {
-            carp("unknown option used: $key - please use only: ".join(", ", keys %{$self->{'allowed_options'}}));
-        }
-        $return->{lc $key} = $opts->{$key};
-    }
+    return unless defined $addr;
+    return unless defined $self->{'peer_by_addr'}->{$addr};
 
-    return($return);
+    return $self->{'peer_by_addr'}->{$addr};
 }
+
+
 ########################################
 
 END {
